package com.persagy.dmp.rwd.dic;

import com.persagy.common.date.DateUtils;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Test;

import java.io.*;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
@Slf4j
public class ImportGroupProjectFuncid {

	private String basepath = "/Users/devsaga/trans";

	private String outPath = "/Users/devsaga/persagy/数据字典20201118/funcid-" + DateUtils.format(new Date()) + ".dml.sql";

	@Test
	public void test1() throws Exception{
		String fileName = "/近期新增需求WD.xlsx";
		String sheetName = "Sheet1";

		String type = "group";
		String groupCode = "WD";
		String projectId = "0";

		List<FuncidDefModel> funcids = new LinkedList<>();
		String classFile = basepath + fileName;
		FileInputStream inputStream = new FileInputStream(classFile);
		Workbook workbook = new XSSFWorkbook(inputStream);
		Sheet sheet = workbook.getSheet(sheetName);

		int startRowNum = 1;
		int lastRowNum = sheet.getLastRowNum();
		for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
			Row row = sheet.getRow(rowIndex);
			if (row == null) {
				log.info("file: {}, sheet: {}, row[{}] is null", fileName, sheetName, rowIndex);
				continue;
			}
			// 0:firstTag, 1:secondTag, 2:name, 3:origCode, 4:unit, 5:origDataType, 6:priorite, 7:imputMode, 8:origDataSource, 9:remark
			// 0系统	1专业 2对象编码 3对象类型 4一级分级标签 5二级分级标签 6属性点 7属性点编码 8单位 9类型 10优先级 11字典选择 12备注
			FuncidDefModel model = new FuncidDefModel();
			model.setType(type);
			model.setGroupCode(groupCode);
			model.setProjectId(projectId);

			String major = (String) ExcelUtils.parseCell(row.getCell(1), "string");
			String classCode = (String) ExcelUtils.parseCell(row.getCell(2), "string");
			model.setClassCode((major == null ? "" : major ) + classCode);

			model.setFirstTag((String) ExcelUtils.parseCell(row.getCell(4), "string"));
			model.setSecondTag((String) ExcelUtils.parseCell(row.getCell(5), "string"));

			String name = (String) ExcelUtils.parseCell(row.getCell(6), "string");
			String code = (String) ExcelUtils.parseCell(row.getCell(7), "string");
			if (code == null) {
				continue;
			}
			name = name.replace("\n", "");
			model.setCode(code.substring(0, 1).toLowerCase() + code.substring(1));
			model.setName(name);
			model.setAliasCode(code);
			model.setAliasName(name);
			model.setOrigCode(code);

			model.setUnit((String) ExcelUtils.parseCell(row.getCell(8), "string"));

			String origDataType = (String) ExcelUtils.parseCell(row.getCell(9), "string");
			model.setOrigDataType(origDataType);
			FuncidDefModel param = DataTypeMapping.getDataType(origDataType);
			if (param == null) {

				System.out.println(JacksonMapper.toSimpleJson(model));
				continue;
			}

			model.setOrigDataType(origDataType);
			model.setCategory(param.getCategory());
			model.setDataType(param.getDataType());
			model.setIsMultiple(param.getIsMultiple());
			model.setIsRegion(param.getIsRegion());
			model.setPriority((String) ExcelUtils.parseCell(row.getCell(10), "string"));

			String origDataSource = (String) ExcelUtils.parseCell(row.getCell(11), "string");
			if (origDataSource != null) {
				origDataSource = origDataSource.replace("\n", "");
			}
			model.setOrigDataSource(origDataSource);
			if (model.getDataType() == FuncidDataType.ENUM || model.getDataType() == FuncidDataType.MENUM || model.getDataType() == FuncidDataType.BOOLEAN) {
				FuncidUtils funcidUtils = new FuncidUtils();
				model.setDataSource(funcidUtils.transEnumDataSource(origDataSource)); // 解析datasource
			}

			String note = (String) ExcelUtils.parseCell(row.getCell(12), "string");
			model.setNote("任晓欣-20201211添加:" + (note == null ? "" : note.replace("\n", "")));
			model.setSubFlag(false);
			model.setWeakPoint(false);
			funcids.add(model);
		}

		for (FuncidDefModel funcid : funcids) {
			String str = toSql(funcid, type, groupCode, projectId);
			System.out.println(str);
		}

	}

	private String toSql(FuncidDefModel model, String type, String groupCode, String projectId) {
		StringBuilder buff = new StringBuilder("");
		buff.append("insert into rwd_def_funcid ");
		buff.append("(id,code,orig_code,name,alias_code,alias_name,class_code,type,group_code,project_id,category,");
		buff.append("first_tag,second_tag,priority,input_mode,unit,data_type,orig_data_type,is_multiple,is_region,formater,data_source,orig_data_source,note,sub_flag,weak_point) ");
		buff.append("values (");
		buff.append("'").append(UUID.randomUUID().toString().replace("-", "")).append("',"); // id
		buff.append("'").append(model.getCode()).append("',"); // code
		buff.append("'").append(model.getOrigCode()).append("',"); // orig_code
		buff.append("'").append(model.getName()).append("',"); // name
		buff.append("'").append(model.getAliasCode()).append("',"); // alias_code
		buff.append("'").append(model.getAliasName()).append("',"); // alias_name
		buff.append("'").append(model.getClassCode()).append("',"); // class_code
		buff.append("'").append(model.getType()).append("',"); // type
		buff.append("'").append(model.getGroupCode()).append("',"); // group_code
		buff.append("'").append(model.getProjectId()).append("',"); // project_id
		buff.append("'").append(model.getCategory().name()).append("',"); // category
		if (model.getFirstTag() != null) {
			buff.append("'").append(model.getFirstTag()).append("',"); // first_tag
		} else {
			buff.append("null,");
		}
		if (model.getSecondTag() != null) {
			buff.append("'").append(model.getSecondTag()).append("',"); // second_tag
		} else {
			buff.append("null,");
		}
		if (model.getPriority() != null) {
			buff.append("'").append(model.getPriority()).append("',"); // priority
		} else {
			buff.append("null,");
		}
		if (model.getInputMode() != null) {
			buff.append("'").append(model.getInputMode()).append("',"); // input_mode
		} else {
			buff.append("null,");
		}
		if (model.getUnit() != null) {
			buff.append("'").append(model.getUnit()).append("',"); // unit
		} else {
			buff.append("null,");
		}

		buff.append("'").append(model.getDataType().name()).append("',"); // data_type
		buff.append("'").append(model.getOrigDataType()).append("',"); // orig_data_type
		buff.append(model.getIsMultiple() ? 1 : 0).append(","); // is_multiple
		buff.append(model.getIsRegion() ? 1 : 0).append(","); // is_region
		if (model.getFormater() != null) {
			buff.append("'").append(model.getFormater()).append("',"); // formater
		} else {
			buff.append("null,");
		}
		if (model.getDataSource() != null) {
			buff.append("'").append(model.getDataSource().toString()).append("',"); // data_source
		} else {
			buff.append("null,");
		}
		if (model.getOrigDataSource() != null) { // orig_data_source
			buff.append("'").append(model.getOrigDataSource()).append("',"); // data_source
		} else {
			buff.append("null,");
		}
		if (model.getNote() != null) {
			buff.append("'").append(model.getNote()).append("',"); // note}
		} else {
			buff.append("null,");
		}
		buff.append(model.getSubFlag() ? 1 : 0).append(","); // sub_flag
		buff.append(model.getWeakPoint() ? 1 : 0).append(","); // weak_point
		buff.deleteCharAt(buff.length() - 1);
		buff.append(");");
		return buff.toString();
	}

	private FuncidDefModel toFuncidDefModel(CacheModel model) {
		FuncidDefModel funcid = new FuncidDefModel();
		funcid.setGroupCode("0");
		funcid.setProjectId("0");
		funcid.setType("common");
		funcid.setClassCode(model.classCode);
		funcid.setFirstTag(model.firstTag != null ? model.firstTag.replace("\n", "") : null);
		funcid.setSecondTag(model.secondTag != null ? model.secondTag.replace("\n", "") : null);
		funcid.setCode(model.origCode.substring(0, 1).toLowerCase() + model.origCode.substring(1));
		funcid.setOrigCode(model.origCode);
		funcid.setAliasCode(model.origCode);
		if (model.name != null) {
			funcid.setName(model.name.replace("\n", ""));
			funcid.setAliasName(model.name.replace("\n", ""));
		} else {
			log.error("funcid name is null : {}", JacksonMapper.toSimpleJson(funcid));
		}
		funcid.setUnit(model.unit);
		funcid.setPriority(model.priority);
		funcid.setInputMode(model.inputMode);
		funcid.setOrigDataType(model.origDataType);
		funcid.setSubFlag(false);
		funcid.setWeakPoint(false);
		FuncidDefModel dataType = DataTypeMapping.getDataType(model.origDataType);
		if (dataType != null) {
			funcid.setCategory(dataType.getCategory());
			funcid.setDataType(dataType.getDataType());
			funcid.setIsMultiple(dataType.getIsMultiple());
			funcid.setIsRegion(dataType.getIsRegion());
		} else {
			return null;
		}
		funcid.setSubFlag(false);
		if (model.remark != null) {
			funcid.setNote(model.remark.replace("\n", ""));
		}

		if (model.origDataSource != null) {
			funcid.setOrigDataSource(model.origDataSource.replace("\n", "nn")); // 去掉换行符
		}
		if (funcid.getDataType() == FuncidDataType.ENUM || funcid.getDataType() == FuncidDataType.MENUM || funcid.getDataType() == FuncidDataType.BOOLEAN) {
			FuncidUtils funcidUtils = new FuncidUtils();
			funcid.setDataSource(funcidUtils.transEnumDataSource(model.origDataSource)); // 解析datasource
		}
		return funcid;
	}

	private void importSpace(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("OtherZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "OTZ其他分区"));
	}

	@Data
	private static class CacheModel {
		private String classCode;
		private String firstTag;
		private String secondTag;
		private String origCode;
		private String name;
		private String unit;
		private String origDataType;
		private String priority; // 优先级
		private String inputMode;
		private String origDataSource; // 字典选择
		private String remark; // 备注
	}

	private List<FuncidDefModel> readSheet(String classCode, String fileName, String sheetName) throws IOException {
		List<FuncidDefModel> data = new LinkedList<>();
		String classFile = basepath + fileName;
		FileInputStream inputStream = new FileInputStream(classFile);
		Workbook workbook = new XSSFWorkbook(inputStream);
		Sheet sheet = workbook.getSheet(sheetName);
		if (sheet == null) {
			return data;
		}

		// 行号从0开始, 从第2行开始读数据
		int startRowNum = 2;
		int lastRowNum = sheet.getLastRowNum();
		CacheModel cache = new CacheModel();
		for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
			Row row = sheet.getRow(rowIndex);
			if (row == null) {
				log.info("file: {}, sheet: {}, row[{}] is null", fileName, sheetName, rowIndex);
				continue;
			}
			// 0:firstTag, 1:secondTag, 2:name, 3:origCode, 4:unit, 5:origDataType, 6:priorite, 7:imputMode, 8:origDataSource, 9:remark
			CacheModel target = new CacheModel();
			target.classCode = classCode;
			target.firstTag = (String) ExcelUtils.parseCell(row.getCell(0), "string");
			target.secondTag = (String) ExcelUtils.parseCell(row.getCell(1), "string");
			target.name = (String) ExcelUtils.parseCell(row.getCell(2), "string");
			target.origCode = (String) ExcelUtils.parseCell(row.getCell(3), "string");
			target.unit = (String) ExcelUtils.parseCell(row.getCell(4), "string");
			target.origDataType = (String) ExcelUtils.parseCell(row.getCell(5), "string");
			target.priority = (String) ExcelUtils.parseCell(row.getCell(6), "string");
			target.inputMode = (String) ExcelUtils.parseCell(row.getCell(7), "string");
			target.origDataSource = (String) ExcelUtils.parseCell(row.getCell(8), "string");
			target.remark = (String) ExcelUtils.parseCell(row.getCell(9), "string");

			if (target.origCode == null || "".equals(target.origCode) || "`".equals(target.origCode)) {
				continue;
			}
			if (FuncidUtils.isBase(target.origCode)) {
				continue;
			}

			// firstTag不为空
			if (target.firstTag != null) {
				cache.firstTag = target.firstTag;
				cache.secondTag = null;
			} else {
				target.firstTag = cache.firstTag;
			}

			// secondTag
			if (target.secondTag != null) {
				cache.secondTag = target.secondTag;
			} else {
				target.secondTag = cache.secondTag;
			}
			FuncidDefModel funcidDefModel = toFuncidDefModel(target);
			if (funcidDefModel == null) {
				log.info("file: {}, sheet: {}, row: {} data type [{}] error", fileName, sheetName, rowIndex, target.origDataType);
			} else {
				data.add(funcidDefModel);
			}
		}
		return data;
	}

}

